package USB;

use strict;
use Carp;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $AUTOLOAD);

require Exporter;
require DynaLoader;
require AutoLoader;

@ISA = qw(Exporter DynaLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(
	USB_CLASS_AUDIO
	USB_CLASS_COMM
	USB_CLASS_DATA
	USB_CLASS_HID
	USB_CLASS_HUB
	USB_CLASS_MASS_STORAGE
	USB_CLASS_PER_INTERFACE
	USB_CLASS_PRINTER
	USB_CLASS_VENDOR_SPEC
	USB_DT_CONFIG
	USB_DT_CONFIG_SIZE
	USB_DT_DEVICE
	USB_DT_DEVICE_SIZE
	USB_DT_ENDPOINT
	USB_DT_ENDPOINT_AUDIO_SIZE
	USB_DT_ENDPOINT_SIZE
	USB_DT_HID
	USB_DT_HUB
	USB_DT_HUB_NONVAR_SIZE
	USB_DT_INTERFACE
	USB_DT_INTERFACE_SIZE
	USB_DT_PHYSICAL
	USB_DT_REPORT
	USB_DT_STRING
	USB_ERROR_BEGIN
	USB_RECIP_DEVICE
	USB_RECIP_ENDPOINT
	USB_RECIP_INTERFACE
	USB_RECIP_OTHER
	USB_REQ_CLEAR_FEATURE
	USB_REQ_GET_CONFIGURATION
	USB_REQ_GET_DESCRIPTOR
	USB_REQ_GET_INTERFACE
	USB_REQ_GET_STATUS
	USB_REQ_SET_ADDRESS
	USB_REQ_SET_CONFIGURATION
	USB_REQ_SET_DESCRIPTOR
	USB_REQ_SET_FEATURE
	USB_REQ_SET_INTERFACE
	USB_REQ_SYNCH_FRAME
	USB_TYPE_CLASS
	USB_TYPE_RESERVED
	USB_TYPE_STANDARD
	USB_TYPE_VENDOR
);
$VERSION = '0.02';

sub AUTOLOAD {
    # This AUTOLOAD is used to 'autoload' constants from the constant()
    # XS function.  If a constant is not found then control is passed
    # to the AUTOLOAD in AutoLoader.

    my $constname;
    ($constname = $AUTOLOAD) =~ s/.*:://;
    croak "& not defined" if $constname eq 'constant';
    my $val = constant($constname, @_ ? $_[0] : 0);
    if ($! != 0) {
	if ($! =~ /Invalid/) {
	    $AutoLoader::AUTOLOAD = $AUTOLOAD;
	    goto &AutoLoader::AUTOLOAD;
	}
	else {
		croak "Your vendor has not defined USB macro $constname";
	}
    }
    no strict 'refs';
    *$AUTOLOAD = sub () { $val };
    goto &$AUTOLOAD;
}

bootstrap USB $VERSION;

# Preloaded methods go here.

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

USB - Perl extension for blah blah blah

=head1 SYNOPSIS

  use USB;
  # Most functions as per libusb

  # Additional to libusb
  $usb_bus = get_usb_busses();
  $usb_bus = get_usb_next_bus($bus);
  $usb_bus = get_usb_prev_bus($bus);
  $busname = get_usb_busname($bus);

  $usb_device = get_usb_devices($bus);
  $usb_device = get_usb_next_device($device);
  $usb_device = get_usb_prev_device($device);
  $devicename = get_usb_devicename($device);

  %device_desc = %{ get_usb_device_descriptor($device) };

=head1 DESCRIPTION

Yeah.. this is very brief..
Check out test.pl to get an idea of how to use the added functions
Everything else should work like libusb specs.
Just be carefull of thoes structure handles.


=head1 Exported constants

  USB_CLASS_AUDIO
  USB_CLASS_COMM
  USB_CLASS_DATA
  USB_CLASS_HID
  USB_CLASS_HUB
  USB_CLASS_MASS_STORAGE
  USB_CLASS_PER_INTERFACE
  USB_CLASS_PRINTER
  USB_CLASS_VENDOR_SPEC
  USB_DT_CONFIG
  USB_DT_CONFIG_SIZE
  USB_DT_DEVICE
  USB_DT_DEVICE_SIZE
  USB_DT_ENDPOINT
  USB_DT_ENDPOINT_AUDIO_SIZE
  USB_DT_ENDPOINT_SIZE
  USB_DT_HID
  USB_DT_HUB
  USB_DT_HUB_NONVAR_SIZE
  USB_DT_INTERFACE
  USB_DT_INTERFACE_SIZE
  USB_DT_PHYSICAL
  USB_DT_REPORT
  USB_DT_STRING
  USB_ERROR_BEGIN
  USB_RECIP_DEVICE
  USB_RECIP_ENDPOINT
  USB_RECIP_INTERFACE
  USB_RECIP_OTHER
  USB_REQ_CLEAR_FEATURE
  USB_REQ_GET_CONFIGURATION
  USB_REQ_GET_DESCRIPTOR
  USB_REQ_GET_INTERFACE
  USB_REQ_GET_STATUS
  USB_REQ_SET_ADDRESS
  USB_REQ_SET_CONFIGURATION
  USB_REQ_SET_DESCRIPTOR
  USB_REQ_SET_FEATURE
  USB_REQ_SET_INTERFACE
  USB_REQ_SYNCH_FRAME
  USB_TYPE_CLASS
  USB_TYPE_RESERVED
  USB_TYPE_STANDARD
  USB_TYPE_VENDOR


=head1 Exported functions

  usb_dev_handle *usb_open(struct usb_device *dev);
  int usb_close(usb_dev_handle *dev);
  int usb_bulk_write(usb_dev_handle *dev, int ep, char *bytes, int size,
	int timeout);
  int usb_bulk_read(usb_dev_handle *dev, int ep, char *bytes, int size,
	int timeout);
  int usb_control_msg(usb_dev_handle *dev, int requesttype, int request,
	int value, int index, char *bytes, int size, int timeout);
  int usb_set_configuration(usb_dev_handle *dev, int configuration);
  int usb_claim_interface(usb_dev_handle *dev, int interface);
  int usb_release_interface(usb_dev_handle *dev, int interface);
  int usb_set_altinterface(usb_dev_handle *dev, int alternate);
  int usb_resetep(usb_dev_handle *dev, unsigned int ep);
  int usb_clear_halt(usb_dev_handle *dev, unsigned int ep);
  int usb_reset(usb_dev_handle *dev);
  char *usb_strerror(void);
  void usb_init(void);
  void usb_set_debug(int level);
  int usb_find_busses(void);
  int usb_find_devices(void);
  struct usb_device *usb_device(usb_dev_handle *dev);


=head1 AUTHOR

A. U. Thor, a.u.thor@a.galaxy.far.far.away

=head1 SEE ALSO

perl(1).

=cut
